/*
 *   Copyright (C) 2007 Marco Martin  <notmart@gmail.com>
 *   derived from DilbertProvider by
 *   Copyright (C) 2007 Tobias Koenig <tokoe@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <QtCore/QDate>
#include <QtCore/QRegExp>
#include <QtCore/QUrl>
#include <QtGui/QImage>
#include <QtNetwork/QHttp>
#include <QtNetwork/QHttpRequestHeader>

#include "pennyarcadeprovider.h"

class PennyArcadeProvider::Private
{
  public:
    Private( PennyArcadeProvider *parent, const QDate &date )
      : mParent( parent ), mDate( date )
    {
      mHttp = new QHttp( "www.penny-arcade.com/comic", 80, mParent );
      connect( mHttp, SIGNAL( done( bool ) ), mParent, SLOT( pageRequestFinished( bool ) ) );
    }

    void referencePageRequestFinished( bool );
    void pageRequestFinished( bool );
    void imageRequestFinished( bool );
    void fetchImage( const QString& );
    void parsePage();

    PennyArcadeProvider *mParent;
    QByteArray mPage;
    QDate mDate;
    QImage mImage;

    QHttp *mHttp;
    QHttp *mReferencePageHttp;
    QHttp *mImageHttp;
};

void PennyArcadeProvider::Private::fetchImage( const QString& data )
{
  //\d{4} is 4 digit year
  const QString pattern( "<img src=\"/images/\\d{4}" );
  const QRegExp exp( pattern );

  //-5 because the \d{4} we've searched plus a /
  int pos = exp.indexIn( data ) + pattern.length() - 5;

  const QString sub = data.mid( pos, data.indexOf( '"', pos ) - pos );

  QUrl url( QString( "http://www.penny-arcade.com/images/%1" ).arg( sub ) );

  mImageHttp = new QHttp( "pennyarcade.com", 80, mParent );
  mImageHttp->setHost( url.host() );
  mImageHttp->get( url.path() );

  mParent->connect( mImageHttp, SIGNAL( done( bool ) ), mParent, SLOT( imageRequestFinished( bool ) ) );
}

void PennyArcadeProvider::Private::referencePageRequestFinished( bool err )
{
  if ( err ) {
    emit mParent->error( mParent );
    return;
  }

  const QString data = QString::fromUtf8( mReferencePageHttp->readAll() );

  fetchImage(data);
}

void PennyArcadeProvider::Private::pageRequestFinished( bool err )
{
  if ( err ) {
    emit mParent->error( mParent );
    return;
  }

  const QString redirPattern( "<meta http-equiv=\"Refresh\" content=\"0;url=" );
  const QRegExp redirExp( redirPattern );

  const QString data = QString::fromUtf8( mHttp->readAll() );

  int pos = redirExp.indexIn( data ) + redirPattern.length();
  const QString sub = data.mid( pos, data.indexOf( '"', pos ) - pos );

  if (pos >= redirPattern.length()) {
      QUrl url( sub );
      mReferencePageHttp = new QHttp( "pennyarcade.com", 80, mParent );
      mReferencePageHttp->setHost( url.host() );
      mReferencePageHttp->get( url.path() );
      connect( mReferencePageHttp, SIGNAL( done( bool ) ), mParent, SLOT( referencePageRequestFinished( bool ) ) );
  }else{
      fetchImage( data );
  }
}

void PennyArcadeProvider::Private::imageRequestFinished( bool error )
{
  if ( error ) {
    emit mParent->error( mParent );
    return;
  }

  QByteArray data = mImageHttp->readAll();
  mImage = QImage::fromData( data );
  emit mParent->finished( mParent );
}

PennyArcadeProvider::PennyArcadeProvider( const QDate &date, QObject *parent )
    : ComicProvider( parent ), d( new Private( this, date ) )
{
    QUrl url;
    if (date == QDate::currentDate()) {
        url = QString( "http://www.penny-arcade.com/comic" );
    }else{
        url = QString( "http://www.penny-arcade.com/comic/%1" ).arg( date.toString( "yyyy/MM/dd" ) );
    }

    QHttpRequestHeader header( "GET", url.path() );
    header.setValue( "User-Agent", "Mozilla/5.0 (compatible; Konqueror/3.5; Linux) KHTML/3.5.6 (like Gecko)" );
    header.setValue( "Accept", "text/html, image/jpeg, image/png, text/*, image/*, */*" );
    header.setValue( "Accept-Encoding", "deflate" );
    header.setValue( "Accept-Charset", "iso-8859-15, utf-8;q=0.5, *;q=0.5" );
    header.setValue( "Accept-Language", "en" );
    header.setValue( "Host", "www.penny-arcade.com" );
    header.setValue( "Connection", "Keep-Alive" );

    d->mHttp->setHost( url.host() );
    d->mHttp->request( header );
}

PennyArcadeProvider::~PennyArcadeProvider()
{
    delete d;
}

QImage PennyArcadeProvider::image() const
{
    return d->mImage;
}

QString PennyArcadeProvider::identifier() const
{
    return QString( "pennyarcade:%1" ).arg( d->mDate.toString( Qt::ISODate ) );
}

#include "pennyarcadeprovider.moc"
